@@ -1,6 +1,6 @@
 /*
  * cwdaemon - morse sounding daemon for the parallel or serial port
- * Copyright (C) 2002 -2004 Joop Stakenborg <pg4i@amsat.org>
+ * Copyright (C) 2002 -2005 Joop Stakenborg <pg4i@amsat.org>
  *                       and many authors, see the AUTHORS file.
  *
  * This program is free oftware; you can redistribute it and/or modify
@@ -52,11 +52,87 @@
 # include <dev/ppbus/ppi.h>
 # include <dev/ppbus/ppbconf.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#if (defined(__unix__) || defined(unix)) && !defined(USG)
+# include <sys/param.h>
+#endif
 
 #include "cwdaemon.h"
 
 /* parport functions */
 
+/*
+ * dev_is_parport(name): check to see whether 'name' is a parallel
+ *     port type character device.  Returns non-zero if the device is
+ *     capable of use for a parallel port based keyer, and zero if it
+ *     is not.  Unfortunately, this is platform specific.
+ */
+
+#if defined(HAVE_LINUX_PPDEV_H)                /* Linux (ppdev) */
+
+int
+dev_is_parport(const char *fname)
+{
+       char nm[MAXPATHLEN];
+       struct stat st;
+       int fd, m;
+
+       m = snprintf(nm, sizeof(nm), "/dev/%s", fname);
+       if (m >= sizeof(nm))
+               return (-1);
+       if ((fd = open(nm, O_RDWR | O_NONBLOCK)) == -1)
+               return (-1);
+       if (fstat(fd, &st) == -1)
+               goto out;
+       if ((st.st_mode & S_IFMT) != S_IFCHR)
+               goto out;
+       if (ioctl(fd, PPGETMODE, &m) == -1)
+               goto out;
+       return (fd);
+out:
+       close(fd);
+       return (-1);
+}
+
+#elif defined(HAVE_DEV_PPBUS_PPI_H)    /* FreeBSD (ppbus/ppi) */
+
+int
+dev_is_parport(const char *fname)
+{
+       char nm[MAXPATHLEN];
+       struct stat st;
+       unsigned char c;
+       int fd, m;
+
+       m = snprintf(nm, sizeof(nm), "/dev/%s", fname);
+       if (m >= sizeof(nm))
+               return (-1);
+       if ((fd = open(nm, O_RDWR | O_NONBLOCK)) == -1)
+               return (-1);
+       if (fstat(fd, &st) == -1)
+               goto out;
+       if ((st.st_mode & S_IFMT) != S_IFCHR)
+               goto out;
+       if (ioctl(fd, PPISSTATUS, &c) == -1)
+               goto out;
+       return (fd);
+out:
+       close(fd);
+       return (-1);
+}
+
+#else                                  /* Fallback (nothing) */
+
+int
+dev_is_parport(const char *fname)
+{
+       return (-1);
+}
+
+#endif
+
 /* Linux wrapper around PPFCONTROL */
 #ifdef HAVE_LINUX_PPDEV_H
 static void
@@ -156,26 +232,12 @@ parport_read_data (int fd)
 
 /* open port and setup ppdev */
 int
-lp_init (cwdevice * dev)
+lp_init (cwdevice * dev, int fd)
 {
-	int fd;
-	char device[MAX_DEVICE];
-
 #ifdef HAVE_LINUX_PPDEV_H
 	int mode;
 #endif
 
-	snprintf (device, MAX_DEVICE - 2, "/dev/%s", dev->desc);
-	device[MAX_DEVICE-1] = '\0';
-	debug ("Opening parallel port [%s]", device);
-
-	fd = open (device, O_RDWR);
-	if (fd < 0)
-	{
-		errmsg ("Opening parallel port %s", dev->desc);
-		exit (1);
-	}
-
 #ifdef HAVE_LINUX_PPDEV_H
 	mode = PARPORT_MODE_PCSPP;
 
@@ -203,13 +265,9 @@ lp_init (cwdevice * dev)
 
 	/* Enable CW & PTT - /STROBE bit (pin 1) */
 	parport_control (fd, PARPORT_CONTROL_STROBE, PARPORT_CONTROL_STROBE);
-
-	debug ("Parallel port [%s] opened", device);
 #endif
 #ifdef HAVE_DEV_PPBUS_PPI_H
 	parport_control (fd, STROBE, STROBE);
-
-	debug ("Parallel port [%s] opened", device);
 #endif
 	dev->fd = fd;
 	dev->reset (dev);
