@@ -1,7 +1,7 @@
 /*
  * cwdaemon - morse sounding daemon for the parallel or serial port
- * Copyright (C) 2002 -2004 Joop Stakenborg <pg4i@amsat.org>
- *                       and many authors, see the AUTHORS file.
+ * Copyright (C) 2002 -2005 Joop Stakenborg <pg4i@amsat.org>
+ *			 and many authors, see the AUTHORS file.
  *
  * This program is free oftware; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,27 +41,52 @@
 #if HAVE_TERMIOS_H
 # include <termios.h>
 #endif
+#if HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#if (defined(__unix__) || defined(unix)) && !defined(USG)
+# include <sys/param.h>
+#endif
 
 #include "cwdaemon.h"
 
 /* serial port functions*/
 
+/*
+ * dev_is_tty(name): check to see whether 'name' is a tty type character
+ *     device capable of TIOCM*.  Returns -1 if the device isn't a
+ *     suitable tty device, and a file descriptor if it is.  This should
+ *     be platform independent.
+ */
 int
-ttys_init (cwdevice * dev)
+dev_is_tty(const char *fname)
+{
+	char nm[MAXPATHLEN];
+	struct stat st;
+	int fd, m;
+
+	m = snprintf(nm, sizeof(nm), "/dev/%s", fname);
+	if (m >= sizeof(nm))
+		return (-1);
+	if ((fd = open(nm, O_RDWR | O_NOCTTY | O_NONBLOCK)) == -1)
+		return (-1);
+	if (fstat(fd, &st) == -1)
+		goto out;
+	if ((st.st_mode & S_IFMT) != S_IFCHR)
+		goto out;
+	if (ioctl(fd, TIOCMGET, &m) == -1)
+		goto out;
+	return (fd);
+out:
+	close(fd);
+	return (-1);
+	
+}
+
+int
+ttys_init (cwdevice * dev, int fd)
 {
-	int fd;
-	char device[MAX_DEVICE];
 
-	snprintf (device, MAX_DEVICE - 2, "/dev/%s", dev->desc);
-	device[MAX_DEVICE-1] = '\0';
-	debug ("Opening serial port [%s]", device);
-	fd = open (device, O_RDWR | O_NOCTTY | O_NDELAY);
-	if (fd < 0)
-	{
-		errmsg ("Open serial port %s", dev->desc);
-		exit (1);
-	}
-	debug ("Serial port [%s] opened", device);
 	dev->fd = fd;
 	dev->reset (dev);
 	return 0;
